#pragma once

// C++ system level
// #include <cstring>     // memset, strncmp
// #include <cstdlib>     // TODO: is this for memcpy?
// #include <functional>

// ESP32 specific
#include "esp_log.h"

// project specific
#include <Types.hpp>

// qthing stuff
// #include <qthing>


namespace SiliconTorch {

  namespace ServiceManager {

    extern const str NVSStateKey;

    enum NVSState : u8 {
      Disabled = 0,
      Enabled  = 1,
      Invalid
    };


    class Service {
      public:

        Service();

        virtual void init()      = 0;

        // internal stuff, only to be called by ServiceManager!
        void postInit();

        virtual void start()     = 0;
        virtual void stop()      = 0;

        const str& getName();
        const str& getNameSpace();

        // virtual str name()       = 0;
        // virtual str nameSpace()  = 0;

        u16 getStartOrder();

        // enabled services will be started in this order. higher = later
        // virtual u16 startOrder();  // defaults to 1337

        bool isEnabled();
        void setEnabled(bool enabled);

      protected:

        void setName(const str& name);
        void setNameSpace(const str& nameSpace);
        void setStartOrder(u16 startOrder);

      private:

        NVSState nvsState = Disabled;

        bool nameSet = false;
        str name = "";

        bool nameSpaceSet = false;
        str nameSpace = "";

        u16 startOrder = 1337;


    };


  }
}
