
# import yaml
import json
import time
import random
import paho.mqtt.client as mqtt


class MMQTT:
  'Minimal MQTT'

  def __init__(self, host: str = 'localhost', port: str = 1883, user: str = None, password: str = '') -> None:

    client_id = 'MinimalMQTT[{:X}]'.format(random.randint(0x100000000000, 0xFFFFFFFFFFFF))
    self.mqtt = mqtt.Client(client_id=client_id, clean_session=True)
    self.mqtt.reconnect_delay_set(1, 1)

    if user is not None:
      self.mqtt.username_pw_set(user, password)


    # We must register an event handler apparently otherwise it won't publish any messages 🙃 

    def _doNothing(*args, **kwargs): pass

    self.mqtt.on_log         = _doNothing
    self.mqtt.on_message     = _doNothing
    self.mqtt.on_publish     = _doNothing
    self.mqtt.on_connect     = _doNothing
    self.mqtt.on_subscribe   = _doNothing
    self.mqtt.on_disconnect  = _doNothing
    self.mqtt.on_unsubscribe = _doNothing


    self.mqtt.connect(host, port)
    self.mqtt.loop_start()

    for X in range(420):
      if self.isConnected():
        return
      else:
        time.sleep(0.1)

    print('WARNING: Not connected after 42 seconds!')

  def publish(self, topic, payload=None, qos=0, retain=False, prettyPrintYAML=False) -> None:
    payloadType = type(payload)

    if issubclass(payloadType, str):
      pl = payload.encode('utf-8')
    elif issubclass(payloadType, (bool, int, float, complex)):
      pl = str(payload).encode('utf-8')
    elif issubclass(payloadType, (list, set, dict)):
      pl = json.dumps(payload).encode('utf-8')
      # pl = yaml.dump(payload, default_flow_style=not prettyPrintYAML).encode('utf-8')
    else:
      pl = payload

    if self.isConnected():
      self.mqtt.publish(topic, pl, qos, retain)

  def isConnected(self) -> bool:
    return self.mqtt.is_connected()
